#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const lodash_1 = require("lodash");
const fs_1 = __importDefault(require("fs"));
const yargs_1 = __importDefault(require("yargs"));
const glob_1 = __importDefault(require("glob"));
const typescript_1 = __importDefault(require("typescript"));
function run() {
    const basePath = getBasePath();
    const baseSwagger = getBaseSwagger();
    const { swaggerComponents } = require(basePath + 'dist/Backend/app.js');
    validateBaseSwagger(baseSwagger);
    baseSwagger.paths = swaggerComponents;
    console.log(JSON.stringify(baseSwagger));
    test();
}
run();
function getBasePath() {
    const currentModuleRelativePath = lodash_1.get(require, 'main.filename');
    if (!currentModuleRelativePath)
        throw new Error('Unable to locate module path');
    const currentModuleAbsolutePath = path_1.default.dirname(currentModuleRelativePath);
    const pathComponents = currentModuleAbsolutePath.split('node_modules');
    const basePath = pathComponents.slice(0, pathComponents.length - 1).join('node_modules');
    return basePath;
}
function getBaseSwagger() {
    const args = yargs_1.default(process.argv).argv;
    const configFileName = args && args.config ? args.config : 'dsapidoc.json';
    const configFile = getBasePath() + configFileName;
    const config = JSON.parse(fs_1.default.readFileSync(configFile).toString());
    return config;
}
function validateBaseSwagger(_baseSwagger) {
}
// testing *********************************************************************
function test() {
    const files = getFiles();
    const compilerOptions = {};
    const program = typescript_1.default.createProgram(files, compilerOptions);
    const nodes = program.getSourceFiles().flatMap(sf => {
        const nodes = [];
        typescript_1.default.forEachChild(sf, node => {
            if (typescript_1.default.isModuleDeclaration(node)) {
                if ((node.flags & typescript_1.default.NodeFlags.Namespace) === 0 && node.body && typescript_1.default.isModuleBlock(node.body)) {
                    node.body.statements.forEach(statement => {
                        nodes.push(statement);
                    });
                    return;
                }
            }
            nodes.push(node);
        });
        return nodes;
    });
    // const responses = {};
    for (const node of nodes) {
        if (node.kind === typescript_1.default.SyntaxKind.ClassDeclaration && getDecorators(node, 'Controller').length > 0) {
            // const decorator = getDecorators(node, 'Controller')[0] as any;
            // const value = getDecoratorValues(decorator);
            // const baseRoute = value[0];
            // '/api/v1/products'
            const methods = node.members.filter((m) => m.kind === typescript_1.default.SyntaxKind.MethodDeclaration);
            for (const method of methods) {
                ['Get', 'Post', 'Put', 'Delete'].map(e => {
                    const decorator = getDecorators(method, e)[0];
                    if (decorator) {
                        // const value = getDecoratorValues(decorator);
                        // const specificRoute = value[0] || '';
                        const responseDecorators = getDecorators(method, 'Response');
                        for (const responseDEcorator of responseDecorators) {
                            // const value = getDecoratorValues(responseDEcorator);
                            if (responseDEcorator.typeArguments && responseDEcorator.typeArguments[0]) {
                                console.log(responseDEcorator.typeArguments[0]);
                            }
                        }
                    }
                });
            }
        }
    }
}
function getFiles() {
    const paths = glob_1.default.sync(getBasePath() + 'src/**/*');
    const filtered = paths.filter(e => e.endsWith('.ts') && !e.endsWith('.d.ts'));
    return filtered;
}
function getDecorators(node, name) {
    const decorators = node.decorators;
    if (!decorators || !decorators.length) {
        return [];
    }
    return decorators
        .filter((decorator) => {
        return lodash_1.get(decorator, 'expression.expression.escapedText') === name
            || lodash_1.get(decorator, 'expression.escapedText') === name;
    });
}
// function getDecoratorValues(decorator: any): string[] {
//   if(decorator && decorator.expression && Array.isArray(decorator.expression.arguments)) {
//     return decorator.expression.arguments.map((e: any) => e.text);
//   }
//   return [];
// };
