"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Response = exports.Parameters = exports.Delete = exports.Put = exports.Post = exports.Get = exports.ControllerMethod = exports.Tags = exports.Controller = exports.methodsMetaKey = exports.controllerMetaKey = void 0;
require("reflect-metadata");
const Errors_1 = require("../Errors");
exports.controllerMetaKey = Symbol('Controller');
exports.methodsMetaKey = Symbol('Method');
function Controller(path) {
    return function (target) {
        registerControllerProperty(target, { basePath: path });
    };
}
exports.Controller = Controller;
function Tags(...args) {
    return function (target) {
        registerControllerProperty(target, { tags: args });
    };
}
exports.Tags = Tags;
function ControllerMethod(method, path) {
    return function validate(target, propertyKey) {
        registerMethodProperty(target, { propertyKey, method, path: path || '' });
    };
}
exports.ControllerMethod = ControllerMethod;
function Get(path) {
    return ControllerMethod('GET', path);
}
exports.Get = Get;
function Post(path) {
    return ControllerMethod('POST', path);
}
exports.Post = Post;
function Put(path) {
    return ControllerMethod('PUT', path);
}
exports.Put = Put;
function Delete(path) {
    return ControllerMethod('DELETE', path);
}
exports.Delete = Delete;
function Parameters(schemas) {
    return function validate(target, propertyKey, descriptor) {
        registerMethodProperty(target, { propertyKey, parameters: schemas });
        let originalMethod = descriptor.value;
        descriptor.value = async function (event, context) {
            const { pathParameters, queryStringParameters, headers, rawBody } = event;
            const { pathParametersSchema, queryStringParametersSchema, headersSchema, bodySchema } = schemas;
            let body;
            if (bodySchema) {
                try {
                    body = typeof rawBody === 'string' && rawBody.trim() ? JSON.parse(rawBody) : {};
                }
                catch (e) {
                    throw new Errors_1.BadRequest('Malformed JSON body');
                }
            }
            try {
                if (pathParametersSchema)
                    await pathParametersSchema.validateAsync(pathParameters);
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In path: ${String(e)}`);
            }
            try {
                if (queryStringParametersSchema)
                    await queryStringParametersSchema.validateAsync(queryStringParameters);
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In querystring: ${String(e)}`);
            }
            try {
                if (headersSchema)
                    await headersSchema.validateAsync(headers);
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In header: ${String(e)}`);
            }
            try {
                if (bodySchema)
                    await bodySchema.validateAsync(body);
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In body: ${String(e)}`);
            }
            return originalMethod.apply(this, [event, context]);
        };
    };
}
exports.Parameters = Parameters;
function Response(_code, _description, _example) {
    return () => { return; };
}
exports.Response = Response;
function registerControllerProperty(target, meta) {
    const properties = Reflect.getMetadata(exports.controllerMetaKey, target.prototype) || {};
    const newProperties = { ...properties, ...meta };
    Reflect.defineMetadata(exports.controllerMetaKey, newProperties, target.prototype);
}
function registerMethodProperty(target, meta) {
    const properties = Reflect.getMetadata(exports.methodsMetaKey, target) || {};
    const currentProperties = properties[meta.propertyKey] || {};
    const newProperties = { ...currentProperties, ...meta };
    properties[meta.propertyKey] = newProperties;
    Reflect.defineMetadata(exports.methodsMetaKey, properties, target);
}
